@@ -11,6 +11,15 @@ import (
 // Row defines a single row in the Table.
 type Row []interface{}
 
+func (r Row) findColumnNumber(colName string) int {
+	for colIdx, col := range r {
+		if fmt.Sprint(col) == colName {
+			return colIdx + 1
+		}
+	}
+	return 0
+}
+
 // RowPainter is a custom function that takes a Row as input and returns the
 // text.Colors{} to use on the entire row
 type RowPainter func(row Row) text.Colors
@@ -308,26 +317,30 @@ func (t *Table) analyzeAndStringify(row Row, hint renderHint) rowStr {
 			t.columnIsNonNumeric[colIdx] = true
 		}
 
-		// convert to a string and store it in the row
-		var colStr string
-		if transformer := t.getColumnTransformer(colIdx, hint); transformer != nil {
-			colStr = transformer(col)
-		} else if colStrVal, ok := col.(string); ok {
-			colStr = colStrVal
-		} else {
-			colStr = fmt.Sprint(col)
-		}
-		if strings.Contains(colStr, "\t") {
-			colStr = strings.Replace(colStr, "\t", "    ", -1)
-		}
-		if strings.Contains(colStr, "\r") {
-			colStr = strings.Replace(colStr, "\r", "", -1)
-		}
-		rowOut[colIdx] = colStr
+		rowOut[colIdx] = t.analyzeAndStringifyColumn(colIdx, col, hint)
 	}
 	return rowOut
 }
 
+func (t *Table) analyzeAndStringifyColumn(colIdx int, col interface{}, hint renderHint) string {
+	// convert to a string and store it in the row
+	var colStr string
+	if transformer := t.getColumnTransformer(colIdx, hint); transformer != nil {
+		colStr = transformer(col)
+	} else if colStrVal, ok := col.(string); ok {
+		colStr = colStrVal
+	} else {
+		colStr = fmt.Sprint(col)
+	}
+	if strings.Contains(colStr, "\t") {
+		colStr = strings.Replace(colStr, "\t", "    ", -1)
+	}
+	if strings.Contains(colStr, "\r") {
+		colStr = strings.Replace(colStr, "\r", "", -1)
+	}
+	return colStr
+}
+
 func (t *Table) getAlign(colIdx int, hint renderHint) text.Align {
 	align := text.AlignDefault
 	if cfg, ok := t.columnConfigMap[colIdx]; ok {
@@ -366,6 +379,48 @@ func (t *Table) getBorderColors(hint renderHint) text.Colors {
 	return t.style.Color.Header
 }
 
+func (t *Table) getBorderLeft(hint renderHint) string {
+	border := t.style.Box.Left
+	if hint.isBorderTop {
+		if t.title != "" {
+			border = t.style.Box.LeftSeparator
+		} else {
+			border = t.style.Box.TopLeft
+		}
+	} else if hint.isBorderBottom {
+		border = t.style.Box.BottomLeft
+	} else if hint.isSeparatorRow {
+		if t.autoIndex && hint.isHeaderOrFooterSeparator() {
+			border = t.style.Box.Left
+		} else if !t.autoIndex && t.shouldMergeCellsVertically(0, hint) {
+			border = t.style.Box.Left
+		} else {
+			border = t.style.Box.LeftSeparator
+		}
+	}
+	return border
+}
+
+func (t *Table) getBorderRight(hint renderHint) string {
+	border := t.style.Box.Right
+	if hint.isBorderTop {
+		if t.title != "" {
+			border = t.style.Box.RightSeparator
+		} else {
+			border = t.style.Box.TopRight
+		}
+	} else if hint.isBorderBottom {
+		border = t.style.Box.BottomRight
+	} else if hint.isSeparatorRow {
+		if t.shouldMergeCellsVertically(t.numColumns-1, hint) {
+			border = t.style.Box.Right
+		} else {
+			border = t.style.Box.RightSeparator
+		}
+	}
+	return border
+}
+
 func (t *Table) getColumnColors(colIdx int, hint renderHint) text.Colors {
 	if t.rowPainter != nil && hint.isRegularRow() && !t.isIndexColumn(colIdx, hint) {
 		colors := t.rowsColors[hint.rowNumber-1]
@@ -448,6 +503,8 @@ func (t *Table) getColumnSeparatorNonBorderNonAutoIndex(mergeCellsAbove bool, me
 		return t.style.Box.MiddleVertical
 	} else if mergeCurrCol {
 		return t.style.Box.LeftSeparator
+	} else if mergeNextCol {
+		return t.style.Box.RightSeparator
 	}
 	return t.style.Box.MiddleSeparator
 }
@@ -491,6 +548,15 @@ func (t *Table) getFormat(hint renderHint) text.Format {
 	return t.style.Format.Row
 }
 
+func (t *Table) getMaxColumnLengthForMerging(colIdx int) int {
+	maxColumnLength := t.maxColumnLengths[colIdx]
+	maxColumnLength += text.RuneCount(t.style.Box.PaddingRight + t.style.Box.PaddingLeft)
+	if t.style.Options.SeparateColumns {
+		maxColumnLength += text.RuneCount(t.style.Box.EmptySeparator)
+	}
+	return maxColumnLength
+}
+
 func (t *Table) getRow(rowIdx int, hint renderHint) rowStr {
 	switch {
 	case hint.isHeaderRow:
@@ -552,6 +618,15 @@ func (t *Table) getVAlign(colIdx int, hint renderHint) text.VAlign {
 	return vAlign
 }
 
+func (t *Table) hasHiddenColumns() bool {
+	for _, cc := range t.columnConfigMap {
+		if cc.Hidden {
+			return true
+		}
+	}
+	return false
+}
+
 func (t *Table) initForRender() {
 	// pick a default style if none was set until now
 	t.Style()
@@ -573,22 +648,13 @@ func (t *Table) initForRender() {
 }
 
 func (t *Table) initForRenderColumnConfigs() {
-	findColumnNumber := func(row Row, colName string) int {
-		for colIdx, col := range row {
-			if fmt.Sprint(col) == colName {
-				return colIdx + 1
-			}
-		}
-		return 0
-	}
-
 	t.columnConfigMap = map[int]ColumnConfig{}
 	for _, colCfg := range t.columnConfigs {
 		// find the column number if none provided; this logic can work only if
 		// a header row is present and has a column with the given name
 		if colCfg.Number == 0 {
 			for _, row := range t.rowsHeaderRaw {
-				colCfg.Number = findColumnNumber(row, colCfg.Name)
+				colCfg.Number = row.findColumnNumber(colCfg.Name)
 				if colCfg.Number > 0 {
 					break
 				}
@@ -601,21 +667,10 @@ func (t *Table) initForRenderColumnConfigs() {
 }
 
 func (t *Table) initForRenderColumnLengths() {
-	var findMaxColumnLengths = func(rows []rowStr) {
-		for _, row := range rows {
-			for colIdx, colStr := range row {
-				longestLineLen := text.LongestLineLen(colStr)
-				if longestLineLen > t.maxColumnLengths[colIdx] {
-					t.maxColumnLengths[colIdx] = longestLineLen
-				}
-			}
-		}
-	}
-
 	t.maxColumnLengths = make([]int, t.numColumns)
-	findMaxColumnLengths(t.rowsHeader)
-	findMaxColumnLengths(t.rows)
-	findMaxColumnLengths(t.rowsFooter)
+	t.parseRowForMaxColumnLengths(t.rowsHeader)
+	t.parseRowForMaxColumnLengths(t.rows)
+	t.parseRowForMaxColumnLengths(t.rowsFooter)
 
 	// restrict the column lengths if any are over or under the limits
 	for colIdx := range t.maxColumnLengths {
@@ -630,22 +685,10 @@ func (t *Table) initForRenderColumnLengths() {
 	}
 }
 
-func (t *Table) initForRenderHideColumns() {
-	// if there is nothing to hide, return fast
-	hasHiddenColumns := false
-	for _, cc := range t.columnConfigMap {
-		if cc.Hidden {
-			hasHiddenColumns = true
-			break
-		}
-	}
-	if !hasHiddenColumns {
-		return
-	}
-
+func (t *Table) hideColumns() map[int]int {
 	colIdxMap := make(map[int]int)
 	numColumns := 0
-	_hideColumns := func(rows []rowStr) []rowStr {
+	hideColumnsInRows := func(rows []rowStr) []rowStr {
 		var rsp []rowStr
 		for _, row := range rows {
 			var rowNew rowStr
@@ -665,13 +708,22 @@ func (t *Table) initForRenderHideColumns() {
 	}
 
 	// hide columns as directed
-	t.rows = _hideColumns(t.rows)
-	t.rowsFooter = _hideColumns(t.rowsFooter)
-	t.rowsHeader = _hideColumns(t.rowsHeader)
+	t.rows = hideColumnsInRows(t.rows)
+	t.rowsFooter = hideColumnsInRows(t.rowsFooter)
+	t.rowsHeader = hideColumnsInRows(t.rowsHeader)
 
 	// reset numColumns to the new number of columns
 	t.numColumns = numColumns
 
+	return colIdxMap
+}
+
+func (t *Table) initForRenderHideColumns() {
+	if !t.hasHiddenColumns() {
+		return
+	}
+	colIdxMap := t.hideColumns()
+
 	// re-create columnIsNonNumeric with new column indices
 	columnIsNonNumeric := make([]bool, t.numColumns)
 	for oldColIdx, nonNumeric := range t.columnIsNonNumeric {
@@ -798,6 +850,17 @@ func (t *Table) isIndexColumn(colIdx int, hint renderHint) bool {
 	return t.indexColumn == colIdx+1 || hint.isAutoIndexColumn
 }
 
+func (t *Table) parseRowForMaxColumnLengths(rows []rowStr) {
+	for _, row := range rows {
+		for colIdx, colStr := range row {
+			longestLineLen := text.LongestLineLen(colStr)
+			if longestLineLen > t.maxColumnLengths[colIdx] {
+				t.maxColumnLengths[colIdx] = longestLineLen
+			}
+		}
+	}
+}
+
 func (t *Table) render(out *strings.Builder) string {
 	outStr := out.String()
 	if t.outputMirror != nil && len(outStr) > 0 {
@@ -893,6 +956,20 @@ func (t *Table) shouldMergeCellsVertically(colIdx int, hint renderHint) bool {
 	return false
 }
 
+func (t *Table) wrapRow(row rowStr) (int, rowStr) {
+	colMaxLines := 0
+	rowWrapped := make(rowStr, len(row))
+	for colIdx, colStr := range row {
+		widthEnforcer := t.columnConfigMap[colIdx].getWidthMaxEnforcer()
+		rowWrapped[colIdx] = widthEnforcer(colStr, t.maxColumnLengths[colIdx])
+		colNumLines := strings.Count(rowWrapped[colIdx], "\n") + 1
+		if colNumLines > colMaxLines {
+			colMaxLines = colNumLines
+		}
+	}
+	return colMaxLines, rowWrapped
+}
+
 // renderHint has hints for the Render*() logic
 type renderHint struct {
 	isAutoIndexColumn bool // auto-index column?
